@@ -37,7 +37,7 @@ else:
     from urllib2 import urlopen #@UnresolvedImport @Reimport
 
 
-__version__ = "0.2.8"
+__version__ = "0.2.9"
 __author__ = "Chris Lucas"
 __contact__ = "chris@chrisjlucas.com"
 __license__ = "MIT"
@@ -54,51 +54,46 @@ class RTorrent:
         self.url = url #: From X{__init__(self, url)}
         self._verbose = _verbose
         self.torrents = [] #: List of L{Torrent} instances
-        self.download_list = [] #: List of torrent info hashes
         self._rpc_methods = [] #: List of rTorrent RPC methods
-        self._p = None #: X{ServerProxy} instance
-        self._connected = False
         self._torrent_cache = []
 
-        self._connect()
-        assert self._connected is True, "rTorrent connection failed"
-        if self._connected:
-            self.client_version_tuple = tuple([int(i) for i in \
-                        self._p.system.client_version().split(".")])
+        assert self._verify_conn(self._get_xmlrpc_conn()) \
+            is True, "rTorrent connection failed"
 
-            self.update()
-            assert self._meets_version_requirement() is True, \
-                "Error: Minimum rTorrent version required is {0}".format(
-                                                    MIN_RTORRENT_VERSION_STR)
-            self.get_torrents()
+        self.client_version_tuple = tuple([int(i) for i in \
+                    self._get_xmlrpc_conn().system.client_version().split(".")])
 
-    def _connect(self):
-        """Create rTorrent connection"""
-        self._connected = False
-        self._p = xmlrpclib.ServerProxy(self.url, verbose=self._verbose)
+        assert self._meets_version_requirement() is True, \
+            "Error: Minimum rTorrent version required is {0}".format(
+                                                MIN_RTORRENT_VERSION_STR)
 
+        self.update()
+        self.get_torrents()
+
+    def _get_xmlrpc_conn(self):
+        """Get ServerProxy instance"""
+        return(xmlrpclib.ServerProxy(self.url, verbose=self._verbose))
+
+    def _verify_conn(self, conn):
+        """Verify given ServerProxy connection is to an rTorrent XMLRPC server"""
         try:
-            # will fail if url given is invalid
-            self._rpc_methods = self._p.system.listMethods()
-            self._connected = True
+            self._rpc_methods = conn.system.listMethods()
         except xmlrpclib.ProtocolError as err:
             sys.stderr.write("*** Exception caught: ProtocolError\n")
             sys.stderr.write("URL: {0}\n".format(err.url))
             sys.stderr.write("Error code: {0}\n".format(err.errcode))
             sys.stderr.write("Error message: {0}\n".format(err.errmsg))
+            return(False)
         except xmlrpclib.ResponseError:
             sys.stderr.write("*** Exception caught: ResponseError")
+            return(False)
 
-    ############### OBSOLETE ###################################################
-    # def _build_method_dict(self):
-    #    self._method_dict = {}
-    #    for method_list in _all_methods_list:
-    #        for m in method_list:
-    #            if m.is_available(self):
-    #                if m.class_name not in self._method_dict.keys():
-    #                    self._method_dict[m.class_name] = []
-    #                self._method_dict[m.class_name].append(m)
-    ############################################################################
+        # simple check, probably sufficient
+        if "system.client_version" not in self._rpc_methods \
+        or "system.library_version" not in self._rpc_methods:
+            return(False)
+        else:
+            return(True)
 
     def _meets_version_requirement(self):
         """Check if rTorrent version is meets requirements"""
@@ -124,7 +119,6 @@ class RTorrent:
         @todo: add validity check for specified view
         """
         self.torrents = []
-        self.download_list = []
         methods = rtorrent.torrent.methods
         retriever_methods = [m for m in methods \
                              if m.is_retriever() and m.is_available(self)]
@@ -141,7 +135,6 @@ class RTorrent:
             for m, r in zip(retriever_methods, result[1:]): # result[0] is the info_hash
                 results_dict[m.varname] = rtorrent.rpc.process_result(m, r)
 
-            self.download_list.append(result[0])
             self.torrents.append(
                     Torrent(self, info_hash=result[0], **results_dict)
             )
@@ -212,6 +205,7 @@ class RTorrent:
         this function doesn't execute instantaneously. If that's what you're
         looking for, use load_torrent_simple() instead.
         """
+        p = self._get_xmlrpc_conn()
         tp = TorrentParser(torrent)
         torrent = xmlrpclib.Binary(tp._raw_torrent)
         info_hash = tp.info_hash
@@ -219,19 +213,19 @@ class RTorrent:
         func_name = self._get_load_function("raw", start, verbose)
 
         # load torrent
-        getattr(self._p, func_name)(torrent)
+        getattr(p, func_name)(torrent)
 
         if verify_load:
             MAX_RETRIES = 3
             i = 0
             while i < MAX_RETRIES:
                 self.get_torrents()
-                if info_hash in self.download_list: break
+                if info_hash in [t.info_hash for t in self.torrents]: break
 
                 time.sleep(1) # was still getting AssertionErrors, delay should help
                 i += 1
 
-            assert info_hash in self.download_list, "Adding torrent was unsuccessful."
+            assert info_hash in [t.info_hash for t in self.torrents], "Adding torrent was unsuccessful."
 
         return(find_torrent(info_hash, self.torrents))
 
@@ -261,6 +255,7 @@ class RTorrent:
         verification that the torrent was successfully added to rTorrent. 
         Use load_torrent() if you would like these features.
         """
+        p = self._get_xmlrpc_conn()
 
         assert file_type in ["raw", "file", "url"], \
             "Invalid file_type, options are: 'url', 'file', 'raw'."
@@ -276,7 +271,7 @@ class RTorrent:
         if file_type in ["raw", "file"]:    finput = xmlrpclib.Binary(torrent)
         elif file_type == "url":            finput = torrent
 
-        getattr(self._p, func_name)(finput)
+        getattr(p, func_name)(finput)
 
     def set_dht_port(self, port):
         """Set DHT port
@@ -535,10 +530,6 @@ methods = [
             """,
             boolean=True,
             ),
-    # just testing
-    Method(RTorrent, 'fake_method', 'get_fake_method'),
-    Method(RTorrent, 'fake_method_dos', 'get_fake_method_dos',
-           min_version=(9, 9, 9)),
 ]
 
 _all_methods_list = [methods,
@@ -549,6 +540,7 @@ _all_methods_list = [methods,
 ]
 
 class_methods_pair = {
+    RTorrent : methods,
     rtorrent.file.File : rtorrent.file.methods,
     rtorrent.torrent.Torrent : rtorrent.torrent.methods,
     rtorrent.tracker.Tracker : rtorrent.tracker.methods,
