@@ -20,10 +20,13 @@
 
 import rtorrent.rpc
 #from rtorrent.rpc import Method
-from rtorrent.peer import Peer
-from rtorrent.tracker import Tracker
-from rtorrent.file import File
+import rtorrent.peer
+import rtorrent.tracker
+import rtorrent.file
 
+Peer = rtorrent.peer.Peer
+Tracker = rtorrent.tracker.Tracker
+File = rtorrent.file.File
 Method = rtorrent.rpc.Method
 
 class Torrent:
@@ -31,7 +34,6 @@ class Torrent:
 
     def __init__(self, _rt_obj, info_hash, **kwargs):
         self._rt_obj = _rt_obj
-        self._p = self._rt_obj._p #: X{ServerProxy} instance
         self.info_hash = info_hash #: info hash for the torrent
         self.rpc_id = self.info_hash #: unique id to pass to rTorrent
         for k in kwargs.keys():
@@ -41,14 +43,15 @@ class Torrent:
         self.trackers = []
         self.files = []
 
-        self._method_list = self._rt_obj._method_dict[self.__class__.__name__]
-        rtorrent.rpc._build_rpc_methods(self, self._method_list)
         self._call_custom_methods()
 
     def __repr__(self):
-        return("<Torrent info_hash=\"{0}\">".format(self.info_hash))
+        return("<Torrent info_hash=\"{0}\" name=\"{1}\">".format(
+                                                            self.info_hash,
+                                                            self.name))
 
     def _call_custom_methods(self):
+        """only calls methods that check instance variables."""
         self._is_hash_checking_queued()
         self._is_started()
         self._is_paused()
@@ -62,11 +65,10 @@ class Torrent:
         @note: also assigns return value to self.peers
         """
         self.peers = []
-        #methods = rtorrent.peer.methods
-        methods = self._rt_obj._method_dict["Peer"]
-        retriever_methods = [m for m in methods if m.is_retriever()]
+        retriever_methods = [m for m in rtorrent.peer.methods \
+                        if m.is_retriever() and m.is_available(self._rt_obj)]
         # need to leave 2nd arg empty (dunno why)
-        m = rtorrent.rpc.Multicall(self._rt_obj)
+        m = rtorrent.rpc.Multicall(self)
         m.add("p.multicall", self.info_hash, "",
                 *[method.rpc_call + "=" for method in retriever_methods])
 
@@ -91,10 +93,11 @@ class Torrent:
         @note: also assigns return value to self.trackers
         """
         self.trackers = []
-        methods = self._rt_obj._method_dict["Tracker"]
-        retriever_methods = [m for m in methods if m.is_retriever()]
+        retriever_methods = [m for m in rtorrent.tracker.methods \
+                        if m.is_retriever() and m.is_available(self._rt_obj)]
+
         # need to leave 2nd arg empty (dunno why)
-        m = rtorrent.rpc.Multicall(self._rt_obj)
+        m = rtorrent.rpc.Multicall(self)
         m.add("t.multicall", self.info_hash, "",
                 *[method.rpc_call + "=" for method in retriever_methods])
 
@@ -120,10 +123,10 @@ class Torrent:
         """
 
         self.files = []
-        methods = self._rt_obj._method_dict["File"]
-        retriever_methods = [m for m in methods if m.is_retriever()]
+        retriever_methods = [m for m in rtorrent.file.methods \
+                        if m.is_retriever() and m.is_available(self._rt_obj)]
         # 2nd arg can be anything, but it'll return all files in torrent regardless
-        m = rtorrent.rpc.Multicall(self._rt_obj)
+        m = rtorrent.rpc.Multicall(self)
         m.add("f.multicall", self.info_hash, "",
                 *[method.rpc_call + "=" for method in retriever_methods])
 
@@ -147,12 +150,6 @@ class Torrent:
             self.files.append(File(self._rt_obj, self.info_hash, \
                                    f_index, **results_dict))
 
-        ### obsolete
-        #offsets = sorted([f.offset for f in self.files])
-        #for f in self.files:
-        #    f.index = offsets.index(f.offset)
-        ###
-
         return(self.files)
 
     def set_directory(self, d):
@@ -162,7 +159,7 @@ class Torrent:
         Also doesn't restart after directory is set, that must be called
         separately.
         """
-        m = rtorrent.rpc.Multicall(self._rt_obj)
+        m = rtorrent.rpc.Multicall(self)
         self.multicall_add(m, "d.try_stop")
         self.multicall_add(m, "d.set_directory", d)
 
@@ -170,7 +167,7 @@ class Torrent:
 
     def start(self):
         """Start the torrent"""
-        m = rtorrent.rpc.Multicall(self._rt_obj)
+        m = rtorrent.rpc.Multicall(self)
         self.multicall_add(m, "d.try_start")
         self.multicall_add(m, "d.is_active")
 
@@ -179,7 +176,7 @@ class Torrent:
 
     def stop(self):
         """"Stop the torrent"""
-        m = rtorrent.rpc.Multicall(self._rt_obj)
+        m = rtorrent.rpc.Multicall(self)
         self.multicall_add(m, "d.try_stop")
         self.multicall_add(m, "d.is_active")
 
@@ -188,7 +185,7 @@ class Torrent:
 
     def close(self):
         """Close the torrent and it's files"""
-        m = rtorrent.rpc.Multicall(self._rt_obj)
+        m = rtorrent.rpc.Multicall(self)
         self.multicall_add(m, "d.close")
 
         return(m.call()[-1])
@@ -197,11 +194,18 @@ class Torrent:
         """Delete the torrent
         
         @note: doesn't delete the downloaded files"""
-        m = rtorrent.rpc.Multicall(self._rt_obj)
+        m = rtorrent.rpc.Multicall(self)
         self.multicall_add(m, "d.erase")
 
         return(m.call()[-1])
 
+    def check_hash(self):
+        """(Re)hash check the torrent"""
+        m = rtorrent.rpc.Multicall(self)
+        self.multicall_add(m, "d.check_hash")
+
+        return(m.call()[-1])
+
     def poll(self):
         """poll rTorrent to get latest peer/tracker/file information"""
         self.get_peers()
@@ -215,20 +219,41 @@ class Torrent:
 
         @return: None
         """
-        multicall = rtorrent.rpc.Multicall(self._rt_obj)
-        retriever_methods = [m for m in self._method_list \
+        multicall = rtorrent.rpc.Multicall(self)
+        retriever_methods = [m for m in methods \
                         if m.is_retriever() and m.is_available(self._rt_obj)]
         for method in retriever_methods:
             multicall.add(method, self.rpc_id)
 
-        result = multicall.call()
-        for m, r in zip(retriever_methods, result):
-            setattr(self, m.varname, rtorrent.rpc.process_result(m, r))
+        multicall.call()
 
         # custom functions (only call private methods, since they only check
         # local variables and are therefore faster)
         self._call_custom_methods()
 
+    def accept_seeders(self, accept_seeds):
+        """Enable/disable whether the torrent connects to seeders
+        
+        @param accept_seeds: enable/disable accepting seeders 
+        @type accept_seeds: bool"""
+        if accept_seeds: call = "d.accepting_seeders.enable"
+        else: call = "d.accepting_seeders.disable"
+
+        m = rtorrent.rpc.Multicall(self)
+        self.multicall_add(m, call)
+
+        return(m.call()[-1])
+
+    def announce(self):
+        """Announce torrent info to tracker(s)"""
+        m = rtorrent.rpc.Multicall(self)
+        self.multicall_add(m, "d.tracker_announce")
+
+        return(m.call()[-1])
+
+    ############################################################################
+    # CUSTOM METHODS (Not part of the official rTorrent API)
+    ############################################################################
     def _is_hash_checking_queued(self):
         """Only checks instance variables, shouldn't be called directly"""
         # if hashing == 3, then torrent is marked for hash checking
@@ -238,27 +263,22 @@ class Torrent:
 
         return(self.hash_checking_queued)
 
-    ############################################################################
-    # CUSTOM METHODS
-    ############################################################################
-
     def is_hash_checking_queued(self):
         """Check if torrent is waiting to be hash checked
         
         @note: Variable where the result for this method is stored Torrent.hash_checking_queued"""
-        m = rtorrent.rpc.Multicall(self._rt_obj)
+        m = rtorrent.rpc.Multicall(self)
         self.multicall_add(m, "d.get_hashing")
         self.multicall_add(m, "d.is_hash_checking")
         results = m.call()
 
-        # this may be handled by Multicall in the future
-        # but for now, just update the variables manually
         setattr(self, "hashing", results[0])
         setattr(self, "hash_checking", results[1])
 
         return(self._is_hash_checking_queued())
 
     def _is_paused(self):
+        """Only checks instance variables, shouldn't be called directly"""
         self.paused = (self.state == 0)
         return(self.paused)
 
@@ -270,6 +290,7 @@ class Torrent:
         return(self._is_paused())
 
     def _is_started(self):
+        """Only checks instance variables, shouldn't be called directly"""
         self.started = (self.state == 1)
         return(self.started)
 
@@ -283,92 +304,123 @@ class Torrent:
 
 methods = [
     # RETRIEVERS
-    Method(Torrent, 'is_hash_checked', 'd.is_hash_checked', None, boolean=True),
-    Method(Torrent, 'is_hash_checking', 'd.is_hash_checking', None, boolean=True),
-    Method(Torrent, 'get_peers_max', 'd.get_peers_max', None),
-    Method(Torrent, 'get_tracker_focus', 'd.get_tracker_focus', None),
-    Method(Torrent, 'get_skip_total', 'd.get_skip_total', None),
-    Method(Torrent, 'get_state', 'd.get_state', None),
-    Method(Torrent, 'get_peer_exchange', 'd.get_peer_exchange', None),
-    Method(Torrent, 'get_down_rate', 'd.get_down_rate', None),
-    Method(Torrent, 'get_connection_seed', 'd.get_connection_seed', None),
-    Method(Torrent, 'get_uploads_max', 'd.get_uploads_max', None),
-    Method(Torrent, 'get_priority_str', 'd.get_priority_str', None),
-    Method(Torrent, 'is_open', 'd.is_open', None, boolean=True),
-    Method(Torrent, 'get_peers_min', 'd.get_peers_min', None),
-    Method(Torrent, 'get_peers_complete', 'd.get_peers_complete', None),
-    Method(Torrent, 'get_tracker_numwant', 'd.get_tracker_numwant', None),
-    Method(Torrent, 'get_connection_current', 'd.get_connection_current', None),
-    Method(Torrent, 'is_complete', 'd.get_complete', None, boolean=True),
-    Method(Torrent, 'get_peers_connected', 'd.get_peers_connected', None),
-    Method(Torrent, 'get_chunk_size', 'd.get_chunk_size', None),
-    Method(Torrent, 'get_state_counter', 'd.get_state_counter', None),
-    Method(Torrent, 'get_base_filename', 'd.get_base_filename', None),
-    Method(Torrent, 'get_state_changed', 'd.get_state_changed', None),
-    Method(Torrent, 'get_peers_not_connected', 'd.get_peers_not_connected', None),
-    Method(Torrent, 'get_directory', 'd.get_directory', None),
-    Method(Torrent, 'is_incomplete', 'd.incomplete', None, boolean=True),
-    Method(Torrent, 'get_tracker_size', 'd.get_tracker_size', None),
-    Method(Torrent, 'is_multi_file', 'd.is_multi_file', None, boolean=True),
-    Method(Torrent, 'get_local_id', 'd.get_local_id', None),
-    Method(Torrent, 'get_ratio', 'd.get_ratio', None,
-           post_process_func=lambda x: x / 1000.0),
-    Method(Torrent, 'get_loaded_file', 'd.get_loaded_file', None),
-    Method(Torrent, 'get_max_file_size', 'd.get_max_file_size', None),
-    Method(Torrent, 'get_size_chunks', 'd.get_size_chunks', None),
-    Method(Torrent, 'is_pex_active', 'd.is_pex_active', None, boolean=True),
-    Method(Torrent, 'get_hashing', 'd.get_hashing', None),
-    Method(Torrent, 'get_bitfield', 'd.get_bitfield', None),
-    Method(Torrent, 'get_local_id_html', 'd.get_local_id_html', None),
-    Method(Torrent, 'get_connection_leech', 'd.get_connection_leech', None),
-    Method(Torrent, 'get_peers_accounted', 'd.get_peers_accounted', None),
-    Method(Torrent, 'get_message', 'd.get_message', None),
-    Method(Torrent, 'is_active', 'd.is_active', None, boolean=True),
-    Method(Torrent, 'get_size_bytes', 'd.get_size_bytes', None),
-    Method(Torrent, 'get_ignore_commands', 'd.get_ignore_commands', None),
-    Method(Torrent, 'get_creation_date', 'd.get_creation_date', None),
-    Method(Torrent, 'get_base_path', 'd.get_base_path', None),
-    Method(Torrent, 'get_left_bytes', 'd.get_left_bytes', None),
-    Method(Torrent, 'get_size_files', 'd.get_size_files', None),
-    Method(Torrent, 'get_size_pex', 'd.get_size_pex', None),
-    Method(Torrent, 'is_private', 'd.is_private', None, boolean=True),
-    Method(Torrent, 'get_max_size_pex', 'd.get_max_size_pex', None),
-    Method(Torrent, 'get_chunks_hashed', 'd.get_chunks_hashed', None),
-    Method(Torrent, 'get_priority', 'd.get_priority', None),
-    Method(Torrent, 'get_skip_rate', 'd.get_skip_rate', None),
-    Method(Torrent, 'get_completed_bytes', 'd.get_completed_bytes', None),
-    Method(Torrent, 'get_name', 'd.get_name', None),
-    Method(Torrent, 'get_completed_chunks', 'd.get_completed_chunks', None),
-    Method(Torrent, 'get_throttle_name', 'd.get_throttle_name', None),
-    Method(Torrent, 'get_free_diskspace', 'd.get_free_diskspace', None),
-    Method(Torrent, 'get_directory_base', 'd.get_directory_base', None),
-    Method(Torrent, 'get_hashing_failed', 'd.get_hashing_failed', None),
-    Method(Torrent, 'get_tied_to_file', 'd.get_tied_to_file', None),
-    Method(Torrent, 'get_down_total', 'd.get_down_total', None),
-    Method(Torrent, 'get_bytes_done', 'd.get_bytes_done', None),
-    Method(Torrent, 'get_up_rate', 'd.get_up_rate', None),
-    Method(Torrent, 'get_up_total', 'd.get_up_total', None),
-
-    # testing
-    Method(Torrent, 'fake_method', 'd.fake_method', None),
+    Method(Torrent, 'is_hash_checked', 'd.is_hash_checked',
+           boolean=True,
+           ),
+    Method(Torrent, 'is_hash_checking', 'd.is_hash_checking',
+           boolean=True,
+           ),
+    Method(Torrent, 'get_peers_max', 'd.get_peers_max'),
+    Method(Torrent, 'get_tracker_focus', 'd.get_tracker_focus'),
+    Method(Torrent, 'get_skip_total', 'd.get_skip_total'),
+    Method(Torrent, 'get_state', 'd.get_state'),
+    Method(Torrent, 'get_peer_exchange', 'd.get_peer_exchange'),
+    Method(Torrent, 'get_down_rate', 'd.get_down_rate'),
+    Method(Torrent, 'get_connection_seed', 'd.get_connection_seed'),
+    Method(Torrent, 'get_uploads_max', 'd.get_uploads_max'),
+    Method(Torrent, 'get_priority_str', 'd.get_priority_str'),
+    Method(Torrent, 'is_open', 'd.is_open',
+           boolean=True,
+           ),
+    Method(Torrent, 'get_peers_min', 'd.get_peers_min'),
+    Method(Torrent, 'get_peers_complete', 'd.get_peers_complete'),
+    Method(Torrent, 'get_tracker_numwant', 'd.get_tracker_numwant'),
+    Method(Torrent, 'get_connection_current', 'd.get_connection_current'),
+    Method(Torrent, 'is_complete', 'd.get_complete',
+           boolean=True,
+           ),
+    Method(Torrent, 'get_peers_connected', 'd.get_peers_connected'),
+    Method(Torrent, 'get_chunk_size', 'd.get_chunk_size'),
+    Method(Torrent, 'get_state_counter', 'd.get_state_counter'),
+    Method(Torrent, 'get_base_filename', 'd.get_base_filename'),
+    Method(Torrent, 'get_state_changed', 'd.get_state_changed'),
+    Method(Torrent, 'get_peers_not_connected', 'd.get_peers_not_connected'),
+    Method(Torrent, 'get_directory', 'd.get_directory'),
+    Method(Torrent, 'is_incomplete', 'd.incomplete',
+           boolean=True,
+           ),
+    Method(Torrent, 'get_tracker_size', 'd.get_tracker_size'),
+    Method(Torrent, 'is_multi_file', 'd.is_multi_file',
+           boolean=True,
+           ),
+    Method(Torrent, 'get_local_id', 'd.get_local_id'),
+    Method(Torrent, 'get_ratio', 'd.get_ratio',
+           post_process_func=lambda x: x / 1000.0,
+           ),
+    Method(Torrent, 'get_loaded_file', 'd.get_loaded_file'),
+    Method(Torrent, 'get_max_file_size', 'd.get_max_file_size'),
+    Method(Torrent, 'get_size_chunks', 'd.get_size_chunks'),
+    Method(Torrent, 'is_pex_active', 'd.is_pex_active',
+           boolean=True,
+           ),
+    Method(Torrent, 'get_hashing', 'd.get_hashing'),
+    Method(Torrent, 'get_bitfield', 'd.get_bitfield'),
+    Method(Torrent, 'get_local_id_html', 'd.get_local_id_html'),
+    Method(Torrent, 'get_connection_leech', 'd.get_connection_leech'),
+    Method(Torrent, 'get_peers_accounted', 'd.get_peers_accounted'),
+    Method(Torrent, 'get_message', 'd.get_message'),
+    Method(Torrent, 'is_active', 'd.is_active',
+           boolean=True,
+           ),
+    Method(Torrent, 'get_size_bytes', 'd.get_size_bytes'),
+    Method(Torrent, 'get_ignore_commands', 'd.get_ignore_commands'),
+    Method(Torrent, 'get_creation_date', 'd.get_creation_date'),
+    Method(Torrent, 'get_base_path', 'd.get_base_path'),
+    Method(Torrent, 'get_left_bytes', 'd.get_left_bytes'),
+    Method(Torrent, 'get_size_files', 'd.get_size_files'),
+    Method(Torrent, 'get_size_pex', 'd.get_size_pex'),
+    Method(Torrent, 'is_private', 'd.is_private',
+           boolean=True,
+           ),
+    Method(Torrent, 'get_max_size_pex', 'd.get_max_size_pex'),
+    Method(Torrent, 'get_num_chunks_hashed', 'd.get_chunks_hashed',
+           aliases=("get_chunks_hashed",)),
+    Method(Torrent, 'get_num_chunks_wanted', 'd.wanted_chunks'),
+    Method(Torrent, 'get_priority', 'd.get_priority'),
+    Method(Torrent, 'get_skip_rate', 'd.get_skip_rate'),
+    Method(Torrent, 'get_completed_bytes', 'd.get_completed_bytes'),
+    Method(Torrent, 'get_name', 'd.get_name'),
+    Method(Torrent, 'get_completed_chunks', 'd.get_completed_chunks'),
+    Method(Torrent, 'get_throttle_name', 'd.get_throttle_name'),
+    Method(Torrent, 'get_free_diskspace', 'd.get_free_diskspace'),
+    Method(Torrent, 'get_directory_base', 'd.get_directory_base'),
+    Method(Torrent, 'get_hashing_failed', 'd.get_hashing_failed'),
+    Method(Torrent, 'get_tied_to_file', 'd.get_tied_to_file'),
+    Method(Torrent, 'get_down_total', 'd.get_down_total'),
+    Method(Torrent, 'get_bytes_done', 'd.get_bytes_done'),
+    Method(Torrent, 'get_up_rate', 'd.get_up_rate'),
+    Method(Torrent, 'get_up_total', 'd.get_up_total'),
+    Method(Torrent, 'is_accepting_seeders', 'd.accepting_seeders',
+           boolean=True,
+           ),
+    Method(Torrent, "get_chunks_seen", "d.chunks_seen",
+           min_version=(0, 9, 1),
+           ),
+    Method(Torrent, "is_partially_done", "d.is_partially_done",
+           boolean=True,
+           ),
+    Method(Torrent, "is_not_partially_done", "d.is_not_partially_done",
+           boolean=True,
+           ),
+    Method(Torrent, "get_time_started", "d.timestamp.started"),
 
     # MODIFIERS
-    Method(Torrent, 'set_uploads_max', 'd.set_uploads_max', None),
-    Method(Torrent, 'set_tied_to_file', 'd.set_tied_to_file', None),
-    Method(Torrent, 'set_tracker_numwant', 'd.set_tracker_numwant', None),
-    Method(Torrent, 'set_custom', 'd.set_custom', None),
-    Method(Torrent, 'set_priority', 'd.set_priority', None),
-    Method(Torrent, 'set_peers_max', 'd.set_peers_max', None),
-    Method(Torrent, 'set_hashing_failed', 'd.set_hashing_failed', None),
-    Method(Torrent, 'set_message', 'd.set_message', None),
-    Method(Torrent, 'set_throttle_name', 'd.set_throttle_name', None),
-    Method(Torrent, 'set_peers_min', 'd.set_peers_min', None),
-    Method(Torrent, 'set_ignore_commands', 'd.set_ignore_commands', None),
-    Method(Torrent, 'set_max_file_size', 'd.set_max_file_size', None),
-    Method(Torrent, 'set_custom5', 'd.set_custom5', None),
-    Method(Torrent, 'set_custom4', 'd.set_custom4', None),
-    Method(Torrent, 'set_custom2', 'd.set_custom2', None),
-    Method(Torrent, 'set_custom1', 'd.set_custom1', None),
-    Method(Torrent, 'set_custom3', 'd.set_custom3', None),
-    Method(Torrent, 'set_connection_current', 'd.set_connection_current', None),
+    Method(Torrent, 'set_uploads_max', 'd.set_uploads_max'),
+    Method(Torrent, 'set_tied_to_file', 'd.set_tied_to_file'),
+    Method(Torrent, 'set_tracker_numwant', 'd.set_tracker_numwant'),
+    Method(Torrent, 'set_custom', 'd.set_custom'),
+    Method(Torrent, 'set_priority', 'd.set_priority'),
+    Method(Torrent, 'set_peers_max', 'd.set_peers_max'),
+    Method(Torrent, 'set_hashing_failed', 'd.set_hashing_failed'),
+    Method(Torrent, 'set_message', 'd.set_message'),
+    Method(Torrent, 'set_throttle_name', 'd.set_throttle_name'),
+    Method(Torrent, 'set_peers_min', 'd.set_peers_min'),
+    Method(Torrent, 'set_ignore_commands', 'd.set_ignore_commands'),
+    Method(Torrent, 'set_max_file_size', 'd.set_max_file_size'),
+    Method(Torrent, 'set_custom5', 'd.set_custom5'),
+    Method(Torrent, 'set_custom4', 'd.set_custom4'),
+    Method(Torrent, 'set_custom2', 'd.set_custom2'),
+    Method(Torrent, 'set_custom1', 'd.set_custom1'),
+    Method(Torrent, 'set_custom3', 'd.set_custom3'),
+    Method(Torrent, 'set_connection_current', 'd.set_connection_current'),
 ]
