@@ -28,7 +28,6 @@ class Tracker:
 
     def __init__(self, _rt_obj, info_hash, **kwargs):
         self._rt_obj = _rt_obj
-        self._p = self._rt_obj._p #: X{ServerProxy} instance
         self.info_hash = info_hash #: info hash for the torrent using this tracker
         for k in kwargs.keys():
             setattr(self, k, kwargs.get(k, None))
@@ -128,9 +127,6 @@ methods = [
            min_version=(0, 9, 0)
            ),
 
-    # testing
-    Method(Tracker, 'fake_method', 't.fake_method'),
-
     # MODIFIERS
     Method(Tracker, 'set_enabled', 't.set_enabled'),
 ]
